// This is the part of MPFRCPP, the C++ interface for MPFR library.

// Copyright (C) 2006 Alexey Beshenov <bav.272304@gmail.com>

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
// MA 02110-1301 USA.

#ifndef __ASSIGNMENT_CONVERSION_CPP
#define __ASSIGNMENT_CONVERSION_CPP

//------------------------------------------------------------
// MPFRCPP ASSIGNMENT/CONVERSION FUNCTIONS
//------------------------------------------------------------

//------------------------------------------------------------
// Changelog:
// 2006-06-12 -- catching errors if number does not fits() in
// conversion target type
//------------------------------------------------------------

//------------------------------------------------------------
// Assignment
//------------------------------------------------------------

void MPFR::operator= (const MPFR& y)
    {
    if ( this->prec() < y.prec()) { this->prec(y.prec()); }
    this->base(y.base());
    mpfr_set (mpc_x, y.mpc_x, mpc_round_mode);
    }


void MPFR::operator= (const unsigned long int y)
    {
    mpfr_set_ui (mpc_x, y, mpc_round_mode);
    }


void MPFR::operator= (const unsigned int y)
    {
    mpfr_set_ui (mpc_x, y, mpc_round_mode);
    }


void MPFR::operator= (const long int y)
    {
    mpfr_set_si (mpc_x, y, mpc_round_mode);
    }


void MPFR::operator= (const int y)
    {
    mpfr_set_si (mpc_x, y, mpc_round_mode);
    }


void MPFR::operator= (const double &y)
    {
    mpfr_set_d (mpc_x, y, mpc_round_mode);
    }


void MPFR::operator= (const long double &y)
    {
    mpfr_set_ld (mpc_x, y, mpc_round_mode);
    }


void MPFR::operator= (const mpz_t& y)
    {
    mpfr_set_z (mpc_x, y, mpc_round_mode);
    }


void MPFR::operator= (const mpq_t& y)
    {
    mpfr_set_q (mpc_x, y, mpc_round_mode);
    }


void MPFR::operator= (const mpf_t& y)
    {
    if ( this->prec() < mpf_get_prec(y)) { this->prec(mpf_get_prec(y)); }
    mpfr_set_f (mpc_x, y, mpc_round_mode);
    }


#ifdef GMP_CPP_INTERFACE
void MPFR::operator= (const mpz_class& y)
    {
    mpfr_set_z (mpc_x, y.get_mpz_t(), mpc_round_mode);
    }


void MPFR::operator= (const mpq_class& y)
    {
    mpfr_set_q (mpc_x, y.get_mpq_t(), mpc_round_mode);
    }


void MPFR::operator= (const mpf_class& y)
    {
    if ( this->prec() < y.get_prec()) { this->prec(y.get_prec()); }
    mpfr_set_f (mpc_x, y.get_mpf_t(), mpc_round_mode);
    }
#endif

void MPFR::operator= (const std::string& s)
    {
    mpfr_set_str(mpc_x, s.c_str(), 0, mpc_round_mode);
    }


void MPFR::operator= (const char *s)
    {
    mpfr_set_str(mpc_x, s, 0, mpc_round_mode);
    }


void MPFR::operator= (const bool y)
    {
    mpfr_set_ui (mpc_x, static_cast<int>(y), mpc_round_mode);
    }


//------------------------------------------------------------
// Conversion
//------------------------------------------------------------

MPFR::operator unsigned long int() const
    {
    if (fits<unsigned int>()) {
        return mpfr_get_ui(mpc_x, mpc_round_mode);
        }
    else { throw error(__FILE__, __LINE__, error::conv_nf); }
    }


MPFR::operator unsigned int() const
    {
    if (fits<unsigned int>()) {
        return mpfr_get_ui(mpc_x, mpc_round_mode);
        }
    else { throw error(__FILE__, __LINE__, error::conv_nf); }
    }


MPFR::operator long int() const
    {
    if (fits<int>()) {
        return mpfr_get_si(mpc_x, mpc_round_mode);
        }
    else { throw error(__FILE__, __LINE__, error::conv_nf); }
    }


MPFR::operator int() const
    {
    if (fits<int>()) {
        return mpfr_get_si(mpc_x, mpc_round_mode);
        }
    else { throw error(__FILE__, __LINE__, error::conv_nf); }
    }


MPFR::operator double () const
    {
    if (fits<double>()) {
        return mpfr_get_d(mpc_x, mpc_round_mode);
        }
    else { throw error(__FILE__, __LINE__, error::conv_nf); }
    }


MPFR::operator long double () const
    {
    if (fits<long double>()) {
        return mpfr_get_ld(mpc_x, mpc_round_mode);
        }
    else { throw error(__FILE__, __LINE__, error::conv_nf); }
    }


#ifdef GMP_CPP_INTERFACE
MPFR::operator mpz_class () const
    {
    mpz_t x;
    mpfr_get_z (x, mpc_x, mpc_round_mode);
    return mpz_class(x);
    }


MPFR::operator mpf_class () const
    {
    mpf_t x;
    mpfr_get_f (x, mpc_x, mpc_round_mode);
    return mpf_class(x);
    }
#endif

MPFR::operator std::string () const
    {
    mp_exp_t exp;
    char * c_str = mpfr_get_str (NULL, &exp, mpc_base, 0, mpc_x, mpc_round_mode);
    if (c_str != NULL) {
        std::ostringstream s;
        s << std::string(c_str).insert(1, 1, '.');
        mpfr_free_str(c_str);
        if (mpfr_number_p (mpc_x) && !mpfr_zero_p (mpc_x)) {
            if (mpc_base <= 10) { s << "e"; }
            else { s << "@"; }
            s << exp-1;
            }
        return s.str();
        }
    else {
        throw error(__FILE__, __LINE__, error::invalid_num_str);
        }
    }

#endif                                            // __ASSIGNMENT_CONVERSION_CPP
