// This is the part of MPFRCPP, the C++ interface for MPFR library.

// Copyright (C) 2006 Alexey Beshenov <bav.272304@gmail.com>

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
// MA 02110-1301 USA.

#ifndef __STREAM_CPP
#define __STREAM_CPP

//------------------------------------------------------------
// MPFRCPP STREAM I/O
//------------------------------------------------------------

//------------------------------------------------------------
// Changelog:
// 2006-06-12 -- catching errors if given number or string is
// invalid
//------------------------------------------------------------

//------------------------------------------------------------
// Stream output
//------------------------------------------------------------

template<typename CharT, class Traits> std::basic_ostream<CharT, Traits>& operator<< (std::basic_ostream<CharT, Traits>& os, const MPFR& x)
    {
    mp_exp_t exp;
    int n = os.precision();

    if (n <= 0) { n = -1; }

// usage:
// os.precision(0) for output with `original' precision
// (consult MPFR documentation)
// os.presision(n) to get n digits after the floating point
// in `scientific' notation.

// For example, n = 3.1415926:

// |------------------------------------------|
// | std::cout.precision() | std::cout<<n     |
// |------------------------------------------|
// |           0           | 3.14159259999... |
// |------------------------------------------|
// |           1           |       3.1        |
// |------------------------------------------|
// |           5           |     3.14159      |
// |-------------------------------------------

// * note that output depends on n.round_mode()

// It differs from operator<< for C floating-point types:
// precision(0) for MPFR means output with `original'
// precision.

// We using n+1 and exp-1, because our format is a fraction
// with an implicit radix point immedately to the RIGHT
// of the first digit.

    char * c_str = mpfr_get_str (NULL, &exp, x.base(), n+1, x.mpc_x, MPFR::frs2mprnd(x.round_mode()));

    std::ostringstream<CharT, Traits> s;

    if (c_str != NULL) {
        if (mpfr_number_p (x.mpc_x)) {
            if (c_str[0] != '-') {
                s << std::string(c_str).insert(1, 1, '.');
                }
            else {
                s << std::string(c_str).insert(2, 1, '.');
                }
            }
        else {
            s << std::string(c_str);
            }

        mpfr_free_str(c_str);
        }
    else {
        throw MPFR::error(__FILE__, __LINE__,  MPFR::error::invalid_num_str);
        }

    if (c_str != NULL) {
        if (mpfr_number_p (x.mpc_x) && !mpfr_zero_p (x.mpc_x)) {
            if (x.base() <= 10) { s << "e"; }
            else { s << "@"; }
            s << exp-1;
            }
        }
    return os << s.str();
    }


//------------------------------------------------------------
// Stream input
//------------------------------------------------------------

template<typename CharT, class Traits> int operator>> (std::basic_istream<CharT, Traits>& is, MPFR& x)
    {
    std::string s;
    is >> s;
// Consult mpfr_strtofr() description in MPFR manual
    if (mpfr_set_str(x.mpc_x, s.c_str(), 0, MPFR::frs2mprnd(x.round_mode())) == -1) {
        throw  MPFR::error(__FILE__, __LINE__, MPFR::error::invalid_num_str);
    }

    }

#endif // __COMPARISON_CPP
